@@ -1,7 +1,7 @@
 import copy
 import itertools
 from functools import partial
-from typing import Dict, Iterable, List, Tuple
+from typing import Iterable, List, Set, Tuple
 
 from isort.format import format_simplified
 
@@ -46,11 +46,15 @@ def sorted_imports(
         sections = base_sections + ("no_sections",)
 
     output: List[str] = []
+    seen_headings: Set[str] = set()
     pending_lines_before = False
     for section in sections:
         straight_modules = parsed.imports[section]["straight"]
         straight_modules = sorting.naturally(
-            straight_modules, key=lambda key: sorting.module_key(key, config, section_name=section)
+            straight_modules,
+            key=lambda key: sorting.module_key(
+                key, config, section_name=section, straight_import=True
+            ),
         )
         from_modules = parsed.imports[section]["from"]
         from_modules = sorting.naturally(
@@ -108,6 +112,8 @@ def sorted_imports(
             comments_above = []
             new_section_output: List[str] = []
             for line in section_output:
+                if not line:
+                    continue
                 if line.startswith("#"):
                     comments_above.append(line)
                 elif comments_above:
@@ -132,7 +138,11 @@ def sorted_imports(
             for line in new_section_output:
                 comments = getattr(line, "comments", ())
                 if comments:
-                    if new_section_output and config.ensure_newline_before_comments:
+                    if (
+                        config.ensure_newline_before_comments
+                        and section_output
+                        and section_output[-1]
+                    ):
                         section_output.append("")
                     section_output.extend(comments)
                 section_output.append(str(line))
@@ -146,7 +156,9 @@ def sorted_imports(
                 continue
 
             section_title = config.import_headings.get(section_name.lower(), "")
-            if section_title:
+            if section_title and section_title not in seen_headings:
+                if config.dedup_headings:
+                    seen_headings.add(section_title)
                 section_comment = f"# {section_title}"
                 if section_comment not in parsed.lines_without_imports[0:1]:
                     section_output.insert(0, section_comment)
@@ -165,6 +177,11 @@ def sorted_imports(
     while output and output[0].strip() == "":
         output.pop(0)
 
+    if config.formatting_function:
+        output = config.formatting_function(
+            parsed.line_separator.join(output), extension, config
+        ).splitlines()
+
     output_at = 0
     if parsed.import_index < parsed.original_line_count:
         output_at = parsed.import_index
@@ -185,7 +202,8 @@ def sorted_imports(
                 line,
                 in_quote="",
                 index=len(formatted_output),
-                section_comments=parsed.section_comments,
+                section_comments=config.section_comments,
+                needs_import=False,
             )
             if not should_skip and line.strip():
                 if (
@@ -213,7 +231,10 @@ def sorted_imports(
             new_out_lines.append(line)
             if line in parsed.import_placements:
                 new_out_lines.extend(parsed.place_imports[parsed.import_placements[line]])
-                if len(formatted_output) <= index or formatted_output[index + 1].strip() != "":
+                if (
+                    len(formatted_output) <= (index + 1)
+                    or formatted_output[index + 1].strip() != ""
+                ):
                     new_out_lines.append("")
         formatted_output = new_out_lines
 
@@ -254,10 +275,10 @@ def _with_from_imports(
         sub_modules = [f"{module}.{from_import}" for from_import in from_imports]
         as_imports = {
             from_import: [
-                f"{from_import} as {as_module}" for as_module in parsed.as_map[sub_module]
+                f"{from_import} as {as_module}" for as_module in parsed.as_map["from"][sub_module]
             ]
             for from_import, sub_module in zip(from_imports, sub_modules)
-            if sub_module in parsed.as_map
+            if sub_module in parsed.as_map["from"]
         }
         if config.combine_as_imports and not ("*" in from_imports and config.combine_star):
             if not config.no_inline_sort:
@@ -266,10 +287,7 @@ def _with_from_imports(
             for from_import in copy.copy(from_imports):
                 if from_import in as_imports:
                     idx = from_imports.index(from_import)
-                    if (
-                        config.keep_direct_and_as_imports
-                        and parsed.imports[section]["from"][module][from_import]
-                    ):
+                    if parsed.imports[section]["from"][module][from_import]:
                         from_imports[(idx + 1) : (idx + 1)] = as_imports.pop(from_import)
                     else:
                         from_imports[idx : (idx + 1)] = as_imports.pop(from_import)
@@ -283,6 +301,11 @@ def _with_from_imports(
                 new_section_output.extend(above_comments)
 
             if "*" in from_imports and config.combine_star:
+                if config.combine_as_imports:
+                    comments = list(comments or ())
+                    comments += parsed.categorized_comments["from"].pop(
+                        f"{module}.__combined_as__", []
+                    )
                 import_statement = wrap.line(
                     with_comments(
                         comments,
@@ -312,10 +335,7 @@ def _with_from_imports(
                             f"{comments and ';' or config.comment_prefix} " f"{comment}"
                         )
                     if from_import in as_imports:
-                        if (
-                            config.keep_direct_and_as_imports
-                            and parsed.imports[section]["from"][module][from_import]
-                        ):
+                        if parsed.imports[section]["from"][module][from_import]:
                             new_section_output.append(
                                 wrap.line(single_import_line, parsed.line_separator, config)
                             )
@@ -343,31 +363,33 @@ def _with_from_imports(
                     from_comments = parsed.categorized_comments["straight"].get(
                         f"{module}.{from_import}"
                     )
-                    if (
-                        config.keep_direct_and_as_imports
-                        and parsed.imports[section]["from"][module][from_import]
-                    ):
+                    if parsed.imports[section]["from"][module][from_import]:
                         new_section_output.append(
-                            with_comments(
-                                from_comments,
-                                wrap.line(
-                                    import_start + from_import, parsed.line_separator, config
+                            wrap.line(
+                                with_comments(
+                                    from_comments,
+                                    import_start + from_import,
+                                    removed=config.ignore_comments,
+                                    comment_prefix=config.comment_prefix,
                                 ),
-                                removed=config.ignore_comments,
-                                comment_prefix=config.comment_prefix,
+                                parsed.line_separator,
+                                config,
                             )
                         )
                     new_section_output.extend(
-                        with_comments(
-                            from_comments,
-                            wrap.line(import_start + as_import, parsed.line_separator, config),
-                            removed=config.ignore_comments,
-                            comment_prefix=config.comment_prefix,
+                        wrap.line(
+                            with_comments(
+                                from_comments,
+                                import_start + as_import,
+                                removed=config.ignore_comments,
+                                comment_prefix=config.comment_prefix,
+                            ),
+                            parsed.line_separator,
+                            config,
                         )
                         for as_import in as_imports[from_import]
                     )
 
-                star_import = False
                 if "*" in from_imports:
                     new_section_output.append(
                         with_comments(
@@ -378,12 +400,9 @@ def _with_from_imports(
                         )
                     )
                     from_imports.remove("*")
-                    star_import = True
                     comments = None
 
                 for from_import in copy.copy(from_imports):
-                    if from_import in as_imports and not config.keep_direct_and_as_imports:
-                        continue
                     comment = (
                         parsed.categorized_comments["nested"].get(module, {}).pop(from_import, None)
                     )
@@ -407,21 +426,21 @@ def _with_from_imports(
                 while from_imports and (
                     from_imports[0] not in as_imports
                     or (
-                        config.keep_direct_and_as_imports
-                        and config.combine_as_imports
+                        config.combine_as_imports
                         and parsed.imports[section]["from"][module][from_import]
                     )
                 ):
                     from_import_section.append(from_imports.pop(0))
-                if star_import:
-                    import_statement = import_start + (", ").join(from_import_section)
-                else:
-                    import_statement = with_comments(
-                        comments,
-                        import_start + (", ").join(from_import_section),
-                        removed=config.ignore_comments,
-                        comment_prefix=config.comment_prefix,
+                if config.combine_as_imports:
+                    comments = (comments or []) + list(
+                        parsed.categorized_comments["from"].pop(f"{module}.__combined_as__", ())
                     )
+                import_statement = with_comments(
+                    comments,
+                    import_start + (", ").join(from_import_section),
+                    removed=config.ignore_comments,
+                    comment_prefix=config.comment_prefix,
+                )
                 if not from_import_section:
                     import_statement = ""
 
@@ -486,11 +505,12 @@ def _with_straight_imports(
             continue
 
         import_definition = []
-        if module in parsed.as_map:
-            if config.keep_direct_and_as_imports and parsed.imports[section]["straight"][module]:
+        if module in parsed.as_map["straight"]:
+            if parsed.imports[section]["straight"][module]:
                 import_definition.append(f"{import_type} {module}")
             import_definition.extend(
-                f"{import_type} {module} as {as_import}" for as_import in parsed.as_map[module]
+                f"{import_type} {module} as {as_import}"
+                for as_import in parsed.as_map["straight"][module]
             )
         else:
             import_definition.append(f"{import_type} {module}")
@@ -527,5 +547,6 @@ def _normalize_empty_lines(lines: List[str]) -> List[str]:
 
 class _LineWithComments(str):
     def __new__(cls, value, comments):
-        cls.comments = comments
-        return super().__new__(cls, value)  # type: ignore
+        instance = super().__new__(cls, value)  # type: ignore
+        instance.comments = comments
+        return instance
