@@ -523,14 +523,13 @@ class Config(_Config):
         self._section_comments = tuple(f"# {heading}" for heading in self.import_headings.values())
         return self._section_comments
 
-    @staticmethod
-    def _parse_known_pattern(pattern: str) -> List[str]:
+    def _parse_known_pattern(self, pattern: str) -> List[str]:
         """Expand pattern if identified as a directory and return found sub packages"""
         if pattern.endswith(os.path.sep):
             patterns = [
                 filename
-                for filename in os.listdir(pattern)
-                if os.path.isdir(os.path.join(pattern, filename))
+                for filename in os.listdir(os.path.join(self.directory, pattern))
+                if os.path.isdir(os.path.join(self.directory, pattern, filename))
             ]
         else:
             patterns = [pattern]
@@ -654,7 +653,9 @@ def _get_config_data(file_path: str, sections: Tuple[str]) -> Dict[str, Any]:
                     float("inf") if max_line_length == "off" else int(max_line_length)
                 )
             settings = {
-                key: value for key, value in settings.items() if key in _DEFAULT_SETTINGS.keys()
+                key: value
+                for key, value in settings.items()
+                if key in _DEFAULT_SETTINGS.keys() or key.startswith(KNOWN_PREFIX)
             }
 
         for key, value in settings.items():
